@@ -17,11 +17,13 @@ import sys
 from scipy.optimize import minimize
 import copy
 
+import scipy.sparse as sp
+import scipy.sparse.linalg as spln
 #===============================================================================
 # The Model
 #===============================================================================
 
-def model(parm, initial, h, attrs, data, sd):
+def model(parm, priors, h, attrs, data, sd):
     """
     Calculate the log probability of a HMF model given data
     
@@ -60,23 +62,24 @@ def model(parm, initial, h, attrs, data, sd):
     # print "parm before: ", parm
     ll = 0
 
-    # First check all values are inside boundaries (if bounds given), and
-    # Get the logprob of the priors-- uniform priors don't count here
-    # We reflect the variables off the edge if we need to
-    i = 0
-    for k, v in initial.iteritems():
-        j = 0
-        if v[1] == "norm":
-
-            ll += norm.logpdf(parm[i], loc=v[2], scale=v[3])
-            print ll
-        elif v[1] == "unif":
-            if parm[i] < v[2] or parm[i] > v[3]:
+    for prior in priors:
+        # A uniform prior doesn't change likelihood but returns -inf if outside bounds
+        if isinstance(prior, Uniform):
+            index = attrs.index(prior.name)
+            if parm[index] < prior.low or parm[index] > prior.high:
                 return -np.inf
-        i += 1
+
+        elif isinstance(prior, Normal):
+            index = attrs.index(prior.name)
+            ll += norm.logpdf(parm[index], loc=prior.mean, scale=prior.sd)
+
+        elif isinstance(prior, MultiNorm):
+            indices = [attrs.index(name) for name in prior.name]
+            ll += _lognormpdf(np.array(parm[indices]), np.array(prior.means),
+                              prior.cov)
 
     # print "parm after:", parm
-    # First rebuild the hod dict from given vals
+    # Rebuild the hod dict from given vals
     hoddict = {attr:val for attr, val in zip(attrs, parm)}
 
     # print h.__dict__
@@ -90,9 +93,9 @@ def model(parm, initial, h, attrs, data, sd):
     ll += np.sum(norm.logpdf(data, loc=model, scale=sd))
 
     # print "got here"
-    return ll, parm
+    return ll
 
-def fit_hod(r, data, sd, initial={}, nwalkers=100, nsamples=100, burnin=10,
+def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10,
            thin=50, nthreads=8, filename=None, **hodkwargs):
     """
     Run an MCMC procedure to fit a model correlation function to data
@@ -109,7 +112,7 @@ def fit_hod(r, data, sd, initial={}, nwalkers=100, nsamples=100, burnin=10,
     sd : array_like
         The uncertainty in the measured correlation function
         
-    initial : dict
+    priors : list of ``Uniform``, ``Gaussian`` or ``MultiGaussian`` objects
         A dictionary with keys as names of parameters in ``parm`` and values
         as a 4-item list. The first item is the guessed value for the parameter,
         the second is a string identifying whether the prior is normal (``norm``)
@@ -117,6 +120,10 @@ def fit_hod(r, data, sd, initial={}, nwalkers=100, nsamples=100, burnin=10,
         upper boundaries or the mean and standard deviation of the normal
         distribution.
         
+    guess : array_like, default ``None``
+        Where to start the chain. If ``None``, will get central values from the
+        distributions.
+        
     nwalkers : int, default 100
         Number of walkers to use for Affine-Invariant Ensemble Sampler
         
@@ -145,22 +152,31 @@ def fit_hod(r, data, sd, initial={}, nwalkers=100, nsamples=100, burnin=10,
     
         
     """
-    if len(initial) == 0:
+
+    if len(priors) == 0:
         raise ValueError("initial must be at least length 1")
 
     # ##NOW TRY A LAPLACE APPROXIMATION TO FIND THE MODE
     # res = minimize()
 
-    # Get the number of variables for MCMC
-    ndim = len(initial)
+
     # Save which attributes are updatable for HOD as a list
-    attrs = [k for k in initial]
+    attrs = []
+    for prior in priors:
+        if isinstance(prior.name, basestring):
+            attrs += [prior.name]
+        else:
+            attrs += prior.name
+
+     # Get the number of variables for MCMC
+    ndim = len(attrs)
 
     # Check that attrs are all applicable
     for a in attrs:
         if a not in ["wdm_mass", "delta_halo", "sigma_8", "n", "omegab", 'omegac',
                      "omegav", "omegak", "H0", "M_1", 'alpha', "M_min", 'gauss_width',
-                     'M_0', 'fca', 'fcb', 'fs', 'delta', 'x', 'ombh2', 'omch2']:
+                     'M_0', 'fca', 'fcb', 'fs', 'delta', 'x', 'omegab_h2',
+                     'omegac_h2', 'h']:
             raise ValueError(a + " is not a valid variable for MCMC in HOD")
 
     # Make sure hodkwargs is ok
@@ -173,22 +189,39 @@ def fit_hod(r, data, sd, initial={}, nwalkers=100, nsamples=100, burnin=10,
 
     # Initialise the HOD object - use all available cpus for this
     h = HOD(r=r, ThreadNum=0, **hodkwargs)
+
     # It's better to get a corr_gal instance now and then the updates are faster
     h.corr_gal
 
     # Now update numthreads for MCMC parallelisation
     h.update(ThreadNum=numthreads)
 
+    # Set guess if not set
+    if len(guess) != len(attrs):
+        guess = []
+        for prior in priors:
+            if isinstance(prior, Uniform):
+                print "uniform"
+                guess += [(prior.high + prior.low) / 2]
+            elif isinstance(prior, Normal):
+                print "norm"
+                guess += [prior.mean]
+            elif isinstance(prior, MultiNorm):
+                print "multinorm"
+                print prior.means
+                guess += prior.means.tolist()
+
     # Get an array of initial values
-    initial_val = np.array([val[0] for k, val in initial.iteritems()])
+    guess = np.array(guess)
+
     # Get an initial value for all walkers, around a small ball near the initial guess
-    stacked_val = initial_val
+    stacked_val = guess.copy()
     for i in range(nwalkers - 1):
-        stacked_val = np.vstack((initial_val, stacked_val))
+        stacked_val = np.vstack((guess, stacked_val))
     p0 = stacked_val * np.random.normal(loc=1.0, scale=0.2, size=ndim * nwalkers).reshape((nwalkers, ndim))
 
     sampler = emcee.EnsembleSampler(nwalkers, ndim, model,
-                                    args=[initial, h, attrs, data, sd],
+                                    args=[priors, h, attrs, data, sd],
                                     threads=nthreads)
 
     # Run a burn-in
@@ -201,3 +234,34 @@ def fit_hod(r, data, sd, initial={}, nwalkers=100, nsamples=100, burnin=10,
     sampler.run_mcmc(pos, nsamples, thin=thin)
 
     return sampler.flatchain, np.mean(sampler.acceptance_fraction)
+
+class Uniform(object):
+    def __init__(self, param, low, high):
+        self.name = param
+        self.low = low
+        self.high = high
+
+class Normal(object):
+    def __init__(self, param, mean, sd):
+        self.name = param
+        self.mean = mean
+        self.sd = sd
+
+class MultiNorm(object):
+    def __init__(self, params, means, cov):
+        self.name = params
+        self.means = means
+        self.cov = cov
+
+def _lognormpdf(x, mu, S):
+    """ Calculate gaussian probability density of x, when x ~ N(mu,sigma) """
+    nx = len(S)
+    tmp = -0.5 * (nx * np.log(2 * np.pi) + np.linalg.slogdet(S)[1])
+
+    err = x - mu
+    if (sp.issparse(S)):
+        numerator = spln.spsolve(S, err).T.dot(err)
+    else:
+        numerator = np.linalg.solve(S, err).T.dot(err)
+
+    return tmp - numerator
