@@ -303,11 +303,6 @@ class HOD(object):
 #===============================================================================
 # Start the actual calculations
 #===============================================================================
-    @property
-    def h(self):
-        """Little h"""
-        return self.pert.cosmo_params['H0'] / 100.0
-
     @property
     def bias(self):
         """A class containing the elements necessary to calculate the halo bias"""
@@ -403,11 +398,13 @@ class HOD(object):
         try:
             return self.__dm_corr
         except:
-            self.__dm_corr = fort.power_to_corr(nlnk=len(self.pert.lnk),
-                                                nr=len(self.r),
-                                                lnk=self.pert.lnk,
-                                                r=self.r,
-                                                power=np.exp(self.matter_power))
+            fit = spline(self.pert.lnk, self.matter_power, k=1)
+            self.__dm_corr = tools.power_to_corr(fit, self.r)
+#             self.__dm_corr = fort.power_to_corr(nlnk=len(self.pert.lnk),
+#                                                 nr=len(self.r),
+#                                                 lnk=self.pert.lnk,
+#                                                 r=self.r,
+#                                                 power=np.exp(self.matter_power))
             return self.__dm_corr
 
     @property
@@ -431,11 +428,13 @@ class HOD(object):
     @property
     def _corr_gal_1h_ss(self):
         """The 1-halo galaxy correlation for sat-sat pairs"""
-        result = fort.power_to_corr(nlnk=len(self.pert.lnk),
-                                    nr=len(self.r),
-                                    lnk=self.pert.lnk,
-                                    r=self.r,
-                                    power=self._power_gal_1h_ss)
+        fit = spline(self.pert.lnk, np.log(self._power_gal_1h_ss))
+        result = tools.power_to_corr(fit, self.r)
+#         result = fort.power_to_corr(nlnk=len(self.pert.lnk),
+#                                     nr=len(self.r),
+#                                     lnk=self.pert.lnk,
+#                                     r=self.r,
+#                                     power=self._power_gal_1h_ss)
         return result
 
     @property
@@ -585,21 +584,24 @@ class HOD(object):
             return self.__corr_gal_2h
         except:
             if not self.scale_dependent_bias and self.halo_exclusion in [None, 'schneider']:
-                self.__corr_gal_2h = fort.power_to_corr(nlnk=len(self.pert.lnk),
-                                                       nr=len(self.r),
-                                                       lnk=self.pert.lnk,
-                                                       r=self.r,
-                                                       power=self.power_gal_2h())
+                fit = spline(self.pert.lnk, np.log(self.power_gal_2h()))
+                self.__corr_gal_2h = tools.power_to_corr(fit, self.r)
+#                 self.__corr_gal_2h = fort.power_to_corr(nlnk=len(self.pert.lnk),
+#                                                        nr=len(self.r),
+#                                                        lnk=self.pert.lnk,
+#                                                        r=self.r,
+#                                                        power=self.power_gal_2h())
             else:
                 self.__corr_gal_2h = np.zeros_like(self.r)
                 for i, r in enumerate(self.r):
                     power = self.power_gal_2h(i)
-
-                    self.__corr_gal_2h[i] = fort.power_to_corr(nlnk=len(self.pert.lnk),
-                                                               nr=len(self.r),
-                                                               lnk=self.pert.lnk,
-                                                               r=self.r,
-                                                               power=power)[0]
+                    fit = spline(self.pert.lnk, np.log(power))
+                    self.__corr_gal_2h[i] = tools.power_to_corr(fit, self.r)
+#                     self.__corr_gal_2h[i] = fort.power_to_corr(nlnk=len(self.pert.lnk),
+#                                                                nr=len(self.r),
+#                                                                lnk=self.pert.lnk,
+#                                                                r=self.r,
+#                                                                power=power)[0]
             return self.__corr_gal_2h
 
     @property
